# Lexicographical Order

A string generator that helps to implement a lexicographical order(lexical order).

This makes reordering transactions faster and simpler. 



## Installing

```yaml
dependencies:
  lexicographical_order: 
```

```dart
import 'package:lexicographical_order/lexicographical_order.dart';
```



# Usage

- **Get a string between two strings in a lexicographical order.**

  ```dart
  final mid = between(prev: 'B', next: 'D');
  final list = [mid, 'D', 'B'];
  assert(listEqauls(list..sort(), ['B', mid, 'D']);
  ```

- **Generates order keys**.

  ```dart
  final keyCount = 100;
  final orderKeys = generateOrderKeys(keyCount);
  ```
  this is useful for generating first order keys.


# Caution

##### **The `between` function accepts only allowed characters as arguments**

more precisely, the following code must return true.

```dart
final prev = prevEntity.orderKey;
final next = nextEntity.orderKey;
return LexOrderValidator().checkBetweenArgs(prev: prev, next: next);
```

The `LexOrderValidator` checks the follwing constraints:

 1.  both `prev` and `next` must be empty or composed of alphabets.

 2.  `next` must not be 'A'

 3.  `prev` and `next` must not be equal to each other.

 4.  if both `prev` and `next` are not empty, `prev` must not precede `next` in the lexicographical order. for example:

     ```dart
     between(prev: 'C', next: 'B');
     ```

     that argument combination is invalid because `between` cannot calculate the string between 'C' and 'B'.

In debug mode, you get an exception if you violate the above constraints. but not in release mode. you can check the constraints manually by using `LexOrderValidator`.





